#include <iostream.h>
#include <string.h>
#include "flat.hh"

#ifdef __sun__
#include "b.hh"
#endif

/*****************************************************************************
**                                 FlatMemory                               **
*****************************************************************************/

FlatMemory::FlatMemory(unsigned int _size)
  : MemoryArea(FLAT_MEMORY), size(_size), data_size(0),
    memory_type(PRIVATE_MEMORY_AREA), used(NULL)
{
  if((address =(void*) new char[_size]) == NULL)
    {
      cerr << "FlatMemory: no requested memory available.\n";
      exit(-1);
    }
}

FlatMemory::FlatMemory(FlatMemory& _mem,unsigned int _size,unsigned int _offset)
  : MemoryArea(FLAT_MEMORY), memory_type(PUBLIC_MEMORY_AREA), used(&_mem)
{
  if((_offset > _mem.DataSize())||(_offset + _size > _mem.DataSize()))
    {
      cerr << "FlatMemory: bad parameters while constructing Flat from Flat.\n";
      exit(-1);
    } 
  address   = (_mem.Address() + _offset);
  size      = _size;
  data_size = _size;
  _mem.AddUse();
}

FlatMemory::FlatMemory(void* _address,unsigned int _size)
  : MemoryArea(FLAT_MEMORY), address(_address), size(_size), data_size(_size),
    memory_type(PUBLIC_MEMORY_AREA), used(NULL)
{
  if(address == NULL)
    {
      cerr << "FlatMemory: NULL address.\n";
      exit(-1);
    }
  //MemoryArea::AddUse();
}


FlatMemory::~FlatMemory()
{
  //cout << "FlatMemory " << (void*) this << ": ~FlatMemory() begin \n"; 

  if(use_count <= 0)
    cerr << "FLATMEMORY: error use count = " << use_count << '\n';
  else if(use_count > 1)
    cerr << "FLATMEMORY: error use count = " << use_count << '\n';
  else
    if(memory_type == PRIVATE_MEMORY_AREA)
      delete (char*) address;    

  if(used != NULL)
    used->RemUse();

  //cout << "FlatMemory: ~FlatMemory() end\n"; 
}

unsigned int FlatMemory::Read(void* _address,unsigned int length,unsigned int offset)
{
  //cerr << "FLAT_MEMORY: Read() at base address: " << (void*) address << '\n';

  if(offset >= size)
    return 0;
  
  if(offset > data_size)
    return 0;

  if(length > (data_size - offset))
    length = size - offset;  

  bcopy((char*)address + offset,(char*)_address,length);
  return length;
}

unsigned int FlatMemory::Write(void* _address,unsigned int length,unsigned int offset)
{
  //cerr << "FLAT_MEMORY: Write() at address: " << (void*) address << '\n';

  if(offset >= size)
    return 0;

  if(length > (size - offset))
    length = size - offset;
   
  if((length + offset) > data_size)
    data_size = length + offset;

  bcopy((char*)_address,(char*)address + offset,length);
  return length;
}

unsigned int FlatMemory::Size()
{
  return size;
}

unsigned int FlatMemory::DataSize()
{
  return data_size;
}

const void* FlatMemory::Address()
{
  return (const void*) address;
}

